#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <proto.h>

#include "gtdisplay.h"
#include "gtconfig.h"
#include "gtcommon.h"
#include "gterror.h"

/************************************************************************\
*
* GtDisplay class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init display.
*
\************************************************************************/
GtDisplay *GtMkDisplay(GtTag *tgs)
{
	GtDisplay *obj;
	
	if(!(obj=new GtDisplay()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtDisplay object!\n");
		return(NULL);
	}
	if(!obj->Init(tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init app.
*
\************************************************************************/
GtDisplay *GtMkDisplay(ulong tag,...)
{
	va_list va;
	GtDisplay *obj;
	int p;
	
	va_start(va,tag);
	p=GtBeginVaTags();
	obj=GtMkDisplay(GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty display.
*
\************************************************************************/
GtDisplay::GtDisplay(void)
{
	dsp=NULL;
	nofreex=FALSE;
}

/************************************************************************\
*
* Destroy display.
*
\************************************************************************/
void GtDisplay::Destroy(void)
{
	if(dsp && !nofreex)
		XCloseDisplay(dsp);
	GtBase::Destroy();
}

/************************************************************************\
*
* Setup display.
*
\************************************************************************/
sbool GtDisplay::Init(GtTag *tgs)
{
	GtTag *t;
	char *dname=NULL;
	
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_DispString:
				dname=(char *)t->d.ptr;
				break;
			case GT_XDisplay:
				dsp=(Display *)t->d.ptr;
				nofreex=TRUE;
				break;
			case GT_DontFreeX:
				if(t->d.val)
					nofreex=TRUE;
				else
					nofreex=FALSE;
				break;
		}
	}
	if(!dsp)
		dsp=XOpenDisplay(dname);
	if(!dsp)
	{
		GtError(this,GTEC_NOXRESOURCE,"No display connection to %s\n",
						(dname ? dname : "<default>"));
		return(FALSE);
	}
	fd=ConnectionNumber(dsp);
	scr=DefaultScreen(dsp);
	return(TRUE);
}

/************************************************************************\
*
* Set display attributes.
*
\************************************************************************/
sbool GtDisplay::Set(GtTag *tgs)
{
	tgs=tgs;
	return(TRUE);
}

/************************************************************************\
*
* Get display attributes.
*
\************************************************************************/
void GtDisplay::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_DispString:
				*ptr=(long)DisplayString(dsp);
				break;
			case GT_XDisplay:
				*ptr=(long)dsp;
				break;
			case GT_DontFreeX:
				*ptr=nofreex;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is a (sub)class of named class.
*
\************************************************************************/
sbool GtDisplay::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtDisplay")==0)
		return(TRUE);
	else
		return(GtBase::IsSubClassOf(cn));
}

/************************************************************************\
*
* Handle one X-event.
*
\************************************************************************/
void GtDisplay::HandleEvent(XEvent *xe)
{
	XNextEvent(dsp,xe);
	if((xe->type==MappingNotify) && ((xe->xmapping.request==MappingModifier) ||
		(xe->xmapping.request==MappingKeyboard)))
		XRefreshKeyboardMapping((XMappingEvent *)xe);
}
